/*
    ReceiveAndSend.cpp

    Serves as a IR remote macro expander
    Receives Samsung32 protocol and on receiving a specified input frame, it sends multiple Samsung32 frames.
    This serves as a Netflix-key emulation for my old Samsung H5273 TV.

    Copyright (C) 2019-2020  Armin Joachimsmeyer
    armin.joachimsmeyer@gmail.com

    This file is part of IRMP https://github.com/ukw100/IRMP.

    IRMP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/gpl.html>.

*/

// ATMEL ATTINY85
// Piezo speaker must have a 270 Ohm resistor in series for USB programming and running at the Samsung TV.
// IR LED has a 270 Ohm resistor in series.
//                                                    +-\/-+
//                                   !RESET (5) PB5  1|    |8  Vcc
// USB+ 3.6V Z-Diode, 1.5kOhm to VCC  Piezo (3) PB3  2|    |7  PB2 (2) TX Debug output
// USB- 3.6V Z-Diode              IR Output (4) PB4  3|    |6  PB1 (1) Feedback LED
//                                              GND  4|    |5  PB0 (0) IR Input
//                                                    +----+
/*
   Pin mapping table for different platforms

   Platform     IR input    IR output   Tone
   -----------------------------------------
   DEFAULT/AVR  3           4           5
   ATtinyX5     0           4           3
   ATtin167     9           8           5 // Digispark pro number schema
   ATtin167     3           2           7
   SAMD21       3           4           5
   ESP8266      14 // D5    12 // D6    %
   ESP32        15          4           %
   BluePill     PA6         PA7         PA3
   APOLLO3      11          12          5
*/

#include <M5StickC.h>
#include <Arduino.h>

/*
   Set library modifiers first to set input and output pin etc.
*/
#include "PinDefinitionsAndMore.h"
#include <irmpSelectAllProtocols.h>  // This enables all possible protocols

#define IRSND_IR_FREQUENCY          38000

#define IRMP_PROTOCOL_NAMES         1 // Enable protocol number mapping to protocol strings - requires some FLASH.
#define IRSND_PROTOCOL_NAMES        1 // Enable protocol number mapping to protocol strings - requires some FLASH.

#define IRMP_SUPPORT_SAMSUNG_PROTOCOL     1
#define IRSND_SUPPORT_SAMSUNG_PROTOCOL    1

/*
   After setting the definitions we can include the code and compile it.
*/

#define USE_ONE_TIMER_FOR_IRMP_AND_IRSND // otherwise we get an error on AVR platform: redefinition of 'void __vector_8()
#include <irmp.c.h>

#include <irsndSelectMain15Protocols.h>
#include <irsnd.c.h>

IRMP_DATA irmp_data;
IRMP_DATA irsnd_data;

void setup()
{
#if defined(MCUSR)
  MCUSR = 0; // To reset old boot flags for next boot
#endif

  Serial.begin(115200);
  delay(2000);
  M5.begin();
  Serial.println("START " __FILE__ " from " __DATE__ " " __TIME__ " Using library version " VERSION_IRMP);

  irmp_init();  
  irmp_irsnd_LEDFeedback(false); // Enable receive signal feedback at LED_BUILTIN for receive and send  
  irsnd_init();

  Serial.print(F("Ready to receive IR signals of protocols: "));
  irmp_print_active_protocols(&Serial);
  Serial.println(F("at pin " STR(IRMP_INPUT_PIN)));
  Serial.println(F("Ready to send IR signals at pin " STR(IRSND_OUTPUT_PIN)));
  Serial.println("setup() done.");
}

void loop()
{
  if (irmp_get_data(&irmp_data)) {
    irmp_result_print(&irmp_data);
    Serial.printf("key detected, 0x%08X", irmp_data.command);
    Serial.println();
    // Flush repeats received
    while (irmp_get_data(&irmp_data))
      yield();
  }

  delay(20);
  
  if (M5.BtnA.wasPressed()) {  // last learned command/obj in irmp_data.
    irsnd_data.protocol   = irmp_data.protocol;
    irsnd_data.address    = irmp_data.address;
    irsnd_data.command    = irmp_data.command;
    irsnd_data.flags      = irmp_data.flags;
    irsnd_send_data(&irsnd_data, true);
    irsnd_data_print(&Serial, &irsnd_data);
  }

  if (M5.BtnB.wasPressed()) { // Oon sound bar toggle power
    irsnd_data.protocol   = IRMP_NEC_PROTOCOL;
    irsnd_data.address    = 0x7F80;
    irsnd_data.command    = 0x12;
    irsnd_data.flags      = 2;
    irsnd_send_data(&irsnd_data, true);
    irsnd_data_print(&Serial, &irsnd_data);
  }

  M5.update(); // refresh button states etc
}
